//
//  AKNSTaskPhotoMachine.m
//  ScreenshotDemo
//
//  Created by Adrian on 1/27/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "AKNSTaskPhotoMachine.h"


@implementation AKNSTaskPhotoMachine

- (id) init 
{
    self = [super init];
    [[NSNotificationCenter defaultCenter] addObserver:self 
            selector:@selector(screenshotTaken:) 
            name:NSTaskDidTerminateNotification 
            object:nil];

    fullScreen = YES;
    sound = YES;
    hide = NO;
	return self;
}

// Based in http://www.cocoabuilder.com/archive/message/cocoa/2005/9/25/146916
// in http://www.cocoadev.com/index.pl?NSTask
// in http://cocoadevcentral.com/articles/000025.php
// and in the manpage of screencapture
- (void) takeScreenshot
{
    NSTask *task;
    task = [[NSTask alloc] init];
    [task setLaunchPath: @"/usr/sbin/screencapture"];

    NSMutableString *param = [[NSMutableString alloc] initWithString:@"-c"]; // send to pasteboard
    if (fullScreen)
    {
        [param appendString:@"S"]; // full screen mode
    }
    else
    {
        [param appendString:@"iW"]; // interactive window mode
    }
    
    if (!sound)
    {
        [param appendString:@"x"]; // disable sounds
    }

    NSArray *arguments;
    arguments = [NSArray arrayWithObjects: [param description], nil];
    [task setArguments: arguments];

    NSPipe *pipe;
    pipe = [NSPipe pipe];
    [task setStandardOutput: pipe];

    if (hide)
    {
        [[NSApplication sharedApplication] hide:self];
    }
    [task launch];
}

// Asynchronous termination of the NSTask:
// when the task is done, grab the image from the pasteboard and return it.
- (void)screenshotTaken:(NSNotification *)aNotification 
{
    if([NSImage canInitWithPasteboard:[NSPasteboard generalPasteboard]])
    {
        screenshot = [[NSImage alloc] initWithPasteboard:[NSPasteboard generalPasteboard]];
    }
    if (hide)
    {
        [[NSApplication sharedApplication] unhide:self];
        [[NSApplication sharedApplication] activateIgnoringOtherApps:YES];
    }
    [[NSNotificationCenter defaultCenter] postNotificationName:AKPhotoHasBeenTaken object:nil];
}

- (void) setFullScreen: (BOOL)value
{
    fullScreen = value;
}

- (void) setSound: (BOOL)value
{
    sound = value;
}

- (void) setMustHide: (BOOL)value
{
    hide = value;
}

- (BOOL) isFullScreen
{
    return fullScreen;
}

- (BOOL) useSound
{
    return sound;
}

- (BOOL) mustHide
{
    return hide;
}

@end
