//
//  AKOpenGLPhotoMachine.m
//  ScreenshotDemo
//
//  Created by Adrian on 1/27/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "AKOpenGLPhotoMachine.h"


@implementation AKOpenGLPhotoMachine

- (void) takeScreenshot
{
    NSBitmapImageRep *_screenCaptureBitmap;
    int bPerPixel, bPerSample, sPerPixel, byPerRow, byPerPixel, w, h;
    NSImage *capture;
    unsigned char *_screenBytes;
    unsigned char *_screenBytesActual;
    NSRect rect = [[[NSScreen screens] objectAtIndex:0] frame];
    NSSize size = rect.size;

    // Gets the screen dimensions:
    w = CGDisplayPixelsWide(kCGDirectMainDisplay);
    h = CGDisplayPixelsHigh(kCGDirectMainDisplay);

    // Fix the destination size so it is not bigger that the source
    // (this means we can resize only to smaller images)
    if (size.height > h)
        size.height = h;

    if (size.width > w)
        size.width = w;

    // Gets the base of the screen memory:
    _screenBytesActual = (unsigned char *)CGDisplayBaseAddress(kCGDirectMainDisplay);

    //need to copy the bytes so we can swap them.
    //if (_screenBytes != NULL)
    //    free (_screenBytes);

    _screenBytes = (unsigned char*)malloc(size.width * size.height * 4);

    // Gets all the screen info:
    bPerPixel = CGDisplayBitsPerPixel(kCGDirectMainDisplay);
    bPerSample = CGDisplayBitsPerSample(kCGDirectMainDisplay);
    sPerPixel =  CGDisplaySamplesPerPixel(kCGDirectMainDisplay);
    byPerRow = CGDisplayBytesPerRow(kCGDirectMainDisplay);
    byPerPixel = bPerPixel / 8;

    if (_screenBytes != 0)
    {
        // Finds how much we need to resize:
        int xSource, ySource, xDestination, yDestination, deltaX, deltaY;

        // the delta steps are:
        deltaX = (((float)w / (float)size.width) + 0.5);
        deltaY = (((float)h / (float)size.height) + 0.5);

        // Copy the screen memory in the new buffer resizing it and "fixing" the pixels:
        for (ySource = 0, yDestination = 0; ySource < h; ySource += deltaY, yDestination++)
        {
            // Pre calucalte this here to save time:

            unsigned long stepSource = ySource * byPerRow;
            unsigned long stepDestination = yDestination * (size.width * 4);

            for (xSource = 0, xDestination = 0; xSource < w; xSource += deltaX, xDestination++)
            {
                unsigned long newPixel;

                if (bPerPixel == 16)
                {
                    unsigned short thisPixel;

                    // Finds the begin of this pixel:
                    thisPixel = *((unsigned short*)(_screenBytesActual + (xSource * byPerPixel) + stepSource));

                    // Transformation is 0xARGB (with 1555) to 0xR0G0B0A0 with 4444
                    newPixel = thisPixel;
                    newPixel =	(((newPixel & 0x8000) >> 15) * 0xF8) 	/* A */ |
                        ((newPixel & 0x7C00) << 17)		/* R */ |
                        ((newPixel & 0x03E0) << 14)		/* G */ |
                        ((newPixel & 0x001F) << 11)		/* B */ ;
                }
                else if (bPerPixel == 32)
                {
                    unsigned long thisPixel;

                    // Finds the beginning of this pixel:
                    thisPixel = *((unsigned long*)(_screenBytesActual + (xSource * byPerPixel) + stepSource));

                    // Transformation is 0xAARRGGBB to 0xRRGGBBAA
                    newPixel = ((thisPixel & 0xFF000000) >> 24) |  ((thisPixel & 0x00FFFFFF) << 8);
                }

                // Sets the new pixel, and just in case check for the postion:
                if ((xDestination < size.width) && (yDestination < size.height))
                    *((unsigned long*)(_screenBytes + (xDestination * 4) + stepDestination)) = newPixel;
            }
        }
    }

    //Phew! create a bitmap w/ the screen capture
    _screenCaptureBitmap = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:&_screenBytes
                                                                   pixelsWide:size.width
                                                                   pixelsHigh:size.height
                                                                bitsPerSample:8
                                                              samplesPerPixel:4
                                                                     hasAlpha:YES
                                                                     isPlanar:NO
                                                               colorSpaceName:NSDeviceRGBColorSpace
                                                                  bytesPerRow:(size.width * 4)
                                                                 bitsPerPixel:32];

    screenshot = [[NSImage alloc] initWithSize:[_screenCaptureBitmap size]];
    [screenshot addRepresentation:_screenCaptureBitmap];

    NSAssert(capture != nil, @"Screen capture failed!");

    [_screenCaptureBitmap release];

    // Notify that we're ready to show the picture
    [[NSNotificationCenter defaultCenter] postNotificationName:AKPhotoHasBeenTaken object:nil];
}

@end
