#import "Controller.h"
#import "AKNSTaskPhotoMachine.h"
#import "AKQuartzPhotoMachine.h"
#import "AKNSWindowHackPhotoMachine.h"
#import "AKOpenGLPhotoMachine.h"
#import "AKQuickDrawPhotoMachine.h"

@implementation Controller

- (id) init
{
    self = [super init];
    
    // Create a default photo machine
    photoMachine = [[AKNSTaskPhotoMachine alloc] init];

    // Be notified by the photomachines that the screenshot has been taken
    [[NSNotificationCenter defaultCenter] addObserver:self 
            selector:@selector(showScreenshot:) 
            name:AKPhotoHasBeenTaken 
            object:nil];

    [saveButton setEnabled:NO];
    [saveMenuItem setEnabled:NO];

    return self;
}

- (void) dealloc
{
    [photoMachine release];
    [super dealloc];
}

- (BOOL) applicationShouldTerminateAfterLastWindowClosed: (NSApplication *)theApplication
{
    return YES;
}

- (IBAction) grabScreenshot: (id)sender
{
    [spinningWheel setHidden:NO];
    [spinningWheel startAnimation:self];
    [photoMachine takeScreenshot];
}

- (void) showScreenshot: (NSNotification *)aNotification
{
    NSImage *image = [photoMachine screenshot];
    if (image)
    {
        [imageView setImage:image];
        [saveButton setEnabled:YES];
        [saveMenuItem setEnabled:YES];
    }
    else
    {
        [imageView setImage:nil];
        [saveButton setEnabled:NO];
        [saveMenuItem setEnabled:NO];
    }
    [spinningWheel setHidden:YES];
}

- (IBAction)saveImage:(id)sender
{
    NSSavePanel *savePanel = [NSSavePanel savePanel];
    int returnCode = [savePanel runModalForDirectory:@"~/Desktop" file:@"screenshot.tiff"];
    if (NSFileHandlingPanelOKButton == returnCode)
    {
        NSData *tiff = [[NSData alloc] initWithData:[[photoMachine screenshot] TIFFRepresentation]];
        [tiff writeToFile:[savePanel filename] atomically:YES];
    }
}

- (IBAction)selectWindowHack:(id)sender
{
    [photoMachine release];
    photoMachine = [[AKNSWindowHackPhotoMachine alloc] init];
    [checkMatrix setEnabled:NO];
}

- (IBAction)selectNSTask:(id)sender
{
    [photoMachine release];
    AKNSTaskPhotoMachine *taskPhotoMachine = [[AKNSTaskPhotoMachine alloc] init];
    [taskPhotoMachine setFullScreen:[checkFullScreen state]];
    [taskPhotoMachine setSound:[checkSound state]];
    [taskPhotoMachine setMustHide:[checkHidden state]];
    
    photoMachine = taskPhotoMachine;
    [checkMatrix setEnabled:YES];
}

- (IBAction)selectOpenGL:(id)sender
{
    [photoMachine release];
    photoMachine = [[AKOpenGLPhotoMachine alloc] init];
    [checkMatrix setEnabled:NO];
}

- (IBAction)selectQuartz:(id)sender
{
    [photoMachine release];
    photoMachine = [[AKQuartzPhotoMachine alloc] init];
    [checkMatrix setEnabled:NO];
}

- (IBAction)selectQuickDraw:(id)sender
{
    [photoMachine release];
    photoMachine = [[AKQuickDrawPhotoMachine alloc] init];
    [checkMatrix setEnabled:NO];
}

- (IBAction)selectFullScreen:(id)sender
{
    NSAssert([photoMachine class] == [AKNSTaskPhotoMachine class], @"Photo Machine of wrong type!");
    [(AKNSTaskPhotoMachine *)photoMachine setFullScreen:[checkFullScreen state]];
}

- (IBAction)selectSound:(id)sender
{
    NSAssert([photoMachine class] == [AKNSTaskPhotoMachine class], @"Photo Machine of wrong type!");
    [(AKNSTaskPhotoMachine *)photoMachine setSound:[checkSound state]];
}

- (IBAction)selectHidden:(id)sender
{
    NSAssert([photoMachine class] == [AKNSTaskPhotoMachine class], @"Photo Machine of wrong type!");
    [(AKNSTaskPhotoMachine *)photoMachine setMustHide:[checkHidden state]];
}

@end
